@@ -312,6 +312,7 @@ objinstptr new_objinst(objinstptr destinst, objinstptr srcinst, int x, int y)
 
 /*--------------------------------------------------------------*/
 /* Generic element destructor function				*/
+/* (Note---this function is not being used anywhere. . .)	*/
 /*--------------------------------------------------------------*/
 
 void remove_element(objinstptr destinst, genericptr genelem)
@@ -323,7 +324,7 @@ void remove_element(objinstptr destinst, genericptr genelem)
    destobject = locdestinst->thisobject;
 
    genelem->type &= REMOVE_TAG;
-   delete_tagged(destobject);
+   delete_tagged(locdestinst);
    calcbboxvalues(locdestinst, (genericptr *)NULL);
    updatepagebounds(destobject);
 }
@@ -715,16 +716,19 @@ Boolean labeltext(int keypressed, char *clientdata)
 	 /* name is unique, change every pin in the matching symbol/	*/
 	 /* schematic to match the new text.				*/
 
-	 if ((curlabel->pin == LOCAL) && (topobject->symschem != NULL)) {
+	 if ((curlabel->pin == LOCAL) && (topobject->symschem != NULL) &&
+			(topobject->symschem->schemtype != PRIMARY)) {
 	    if ((findlabelcopy(curlabel, labelbuf) == NULL)
 			&& (findlabelcopy(curlabel, curlabel->string) == NULL)) {
-	       changeotherpins(curlabel, labelbuf);
-	       if (topobject->schemtype == PRIMARY || topobject->schemtype == SECONDARY)
-	          Wprintf("Changed corresponding pin in associated symbol");
-	       else
-	          Wprintf("Changed corresponding pin in associated schematic");
-	       incr_changes(topobject->symschem);
-	       invalidate_netlist(topobject->symschem);
+	       if (changeotherpins(curlabel, labelbuf) > 0) {
+	          if (topobject->schemtype == PRIMARY ||
+				topobject->schemtype == SECONDARY)
+	             Wprintf("Changed corresponding pin in associated symbol");
+	          else
+	             Wprintf("Changed corresponding pin in associated schematic");
+	          incr_changes(topobject->symschem);
+	          invalidate_netlist(topobject->symschem);
+	       }
 	    }
 	 }
 	
@@ -1901,6 +1905,7 @@ void join()
    short     *selectobj;
    polyptr   *newpoly, nextwire;
    pathptr   *newpath;
+   genericptr *pgen;
    short     *scount, *sptr, *sptr2, *direc, *order;
    short     ordered, startpt = 0;
    short     numpolys, numlabels, numpoints, polytype;
@@ -2049,9 +2054,6 @@ void join()
       return;
    }
 
-   XcSetFunction(GXcopy);
-   XSetForeground(dpy, areawin->gc, BACKGROUND);
-
    /* create the new polygon or path */
 
    if (allpolys) {
@@ -2101,9 +2103,6 @@ void join()
       register_for_undo(XCF_Delete, UNDO_DONE, areawin->topinstance,
                 delobj, NORMAL);
 
-      XcSetForeground((*newpoly)->color);
-      UDrawPolygon(*newpoly);
-
    }
    else {	/* create a path */
 
@@ -2114,18 +2113,37 @@ void join()
       (*newpath)->parts = 0;
       (*newpath)->plist = (genericptr *) malloc(sizeof(genericptr));
       (*newpath)->passed = NULL;
+      topobject->parts++;
 
-      /* move the elements from the top level into the path structure */
+      /* copy the elements from the top level into the path structure */
 
       for (sptr = order; sptr < order + numpolys; sptr++) {
 	 genericptr *oldelem = topobject->plist + *sptr;
 	 genericptr *newelem;
 
+	 switch (ELEMENTTYPE(*oldelem)) {
+	    case POLYGON: {
+	       polyptr copypoly = TOPOLY(oldelem);
+	       polyptr *newpoly;
+	       NEW_POLY(newpoly, (*newpath));
+	       polycopy(*newpoly, copypoly);
+	    } break;
+	    case ARC: {
+	       arcptr copyarc = TOARC(oldelem);
+	       arcptr *newarc;
+	       NEW_ARC(newarc, (*newpath));
+	       arccopy(*newarc, copyarc);
+	    } break;
+	    case SPLINE: {
+	       splineptr copyspline = TOSPLINE(oldelem);
+	       splineptr *newspline;
+	       NEW_SPLINE(newspline, (*newpath));
+	       splinecopy(*newspline, copyspline);
+	    } break;
+	 }
+
 	 (*newpath)->parts++;
-         (*newpath)->plist = (genericptr *) realloc((*newpath)->plist,
-			(*newpath)->parts * sizeof(genericptr));
 	 newelem = (*newpath)->plist + (*newpath)->parts - 1;
-	 *newelem = *oldelem;
 
 	 /* reverse point order if necessary */
 
@@ -2149,17 +2167,25 @@ void join()
 	    decomposearc(*newpath);
       }
 
-      for (sptr = scount; sptr < scount + numpolys; sptr++) {
-	 easydraw(*sptr, DOFORALL);
-	 removep(sptr, 1);  /* close up list but do not delete elements */
-	 /* revise the list of path elements */
-	 for (sptr2 = sptr + 1; sptr2 < scount + numpolys; sptr2++)
-	    if (*sptr2 > *sptr) (*sptr2)--;
+      /* delete the old elements from the list */
+
+      register_for_undo(XCF_Join, UNDO_MORE, areawin->topinstance, *newpath);
+
+      delobj = delete_element(areawin->topinstance, scount, numpolys, NORMAL);
+
+      register_for_undo(XCF_Delete, UNDO_DONE, areawin->topinstance,
+                delobj, NORMAL);
+
+      /* Remove the path parts from the selection list and add the path */
+      clearselects();
+      selectobj = allocselect();
+      for (pgen = topobject->plist; pgen < topobject->plist + topobject->parts;
+		pgen++) {
+	 if ((TOPATH(pgen)) == (*newpath)) {
+	    *selectobj = (short)(pgen - topobject->plist);
+	    break;
+	 }
       }
-      
-      XcSetForeground((*newpath)->color);
-      UDrawPath(*newpath);
-      topobject->parts++;
    }
 
    /* clean up */
@@ -2169,7 +2195,6 @@ void join()
    /* other actions that clear only when something has not been	*/
    /* preselected before the action.  Elements must be selected	*/
    /* prior to the "join" action, by necessity.			*/
-   /* clearselects(); */
    free(scount);
    free(order);
    free(direc);
