@@ -7,6 +7,7 @@
 
 /* Note variable argument list for register_for_undo() */
 extern void register_for_undo(u_int, u_char, objinstptr, ...);
+extern void undo_finish_series(void);
 extern void undo_action(void);
 extern void redo_action(void);
 extern void flush_undo_stack(void);
@@ -38,7 +39,7 @@ extern Tcl_Obj *evaluate_raw(objectptr, oparamptr, objinstptr, int *);
 extern char *TCL_to_PS(char *);
 extern XCWindowData *GUI_init(int, Tcl_Obj *CONST objv[]);
 extern void build_app_database(Tk_Window);
-extern int XcFakeTagCall(Tcl_Interp *, int, ...);
+extern int XcInternalTagCall(Tcl_Interp *, int, ...);
 extern char *translateparamtype(int);
 extern char *translatestyle(int);
 extern char *translateencoding(int);
@@ -219,7 +220,7 @@ extern void polycopy(polyptr, polyptr);
 extern void splinecopy(splineptr, splineptr);
 extern void pathcopy(pathptr, pathptr);
 extern void instcopy(objinstptr, objinstptr);
-extern void delete_tagged(objectptr);
+extern void delete_tagged(objinstptr);
 extern void createcopies(void);
 extern void copydrag(void);
 extern void copy_op(int, int, int);
@@ -274,6 +275,7 @@ extern void varcheck(FILE *, short, objectptr, short *, genericptr, u_char);
 extern void varfcheck(FILE *, float, objectptr, short *, genericptr, u_char);
 extern void varpcheck(FILE *, short, objectptr, int, short *, genericptr, u_char);
 extern void getfile(xcWidget, pointertype, caddr_t);
+extern int filecmp(char *, char *);
 extern Boolean nextfilename(void);
 extern FILE *libopen(char *, short, char *, int);
 extern void loadfontlib(void);
@@ -641,7 +643,7 @@ extern void addnewfont(xcWidget, caddr_t, caddr_t);
 #ifdef TCL_WRAPPER
 extern Tcl_Obj *tclglobals(objinstptr);
 extern Tcl_Obj *tcltoplevel(objinstptr);
-void ratsnest(objectptr);
+void ratsnest(objinstptr);
 #endif
 
 extern void ReferencePosition(objinstptr, XPoint *, XPoint *);
@@ -858,12 +860,12 @@ extern void dopintype(xcWidget, pointertype, caddr_t);
 extern void setsymschem(void);
 extern int findpageobj(objectptr);
 extern void collectsubschems(int);
-extern int findsubschems(int, objectptr, int, short *);
+extern int findsubschems(int, objectptr, int, short *, Boolean);
 extern void copypinlabel(labelptr);
 extern int checkschem(objectptr, char *);
 extern int checksym(objectptr, char *);
-extern void changeotherpins(labelptr, stringpart *);
-extern void swapschem(int, int);
+extern int changeotherpins(labelptr, stringpart *);
+extern void swapschem(int, int, char *);
 extern void dobeforeswap(xcWidget, caddr_t, caddr_t);
 extern void schemdisassoc(void);
 extern void startschemassoc(xcWidget, pointertype, caddr_t);
@@ -886,7 +888,7 @@ extern void setoptionmenu(void);
 extern int test_insideness(int, int, XPoint *);
 extern Boolean pathselect(genericptr *, short, float);
 extern Boolean areaelement(genericptr *);
-extern void selectarea(void);
+extern Boolean selectarea(objectptr);
 extern void startdesel(xcWidget, caddr_t, caddr_t);
 extern void deselect(xcWidget, caddr_t, caddr_t);
 extern void draw_normal_selected(objectptr, objinstptr);
