@@ -563,13 +563,15 @@ Boolean areaelement(genericptr *curgen)
 /* select all objects from a defined box area */
 /*--------------------------------------------*/
 
-void selectarea()
+Boolean selectarea(objectptr selobj)
 {
    short	*newselect;
    genericptr   *curgen, *pathgen;
    Boolean	selected;
    stringpart	*strptr;
-   int		locpos;
+   int		locpos, cx, cy;
+   objinstptr	curinst;
+   XPoint	saveorig, savesave, tmppt;
 
    /* put points of area bounding box into proper order */
 
@@ -585,22 +587,45 @@ void selectarea()
       areawin->origin.x = areawin->save.x;
       areawin->save.x = tmp;
    }
+   if (selobj == topobject) {
+      areawin->textpos = areawin->textend = 0;
+   } 
 
-   areawin->textpos = areawin->textend = 0;
-   for (curgen = topobject->plist; curgen < topobject->plist
-	+ topobject->parts; curgen++) {
+   for (curgen = selobj->plist; curgen < selobj->plist + selobj->parts; curgen++) {
 
       /* apply the selection filter */
       if (!((*curgen)->type & areawin->filter)) continue;
 
       switch(ELEMENTTYPE(*curgen)) {
 	case(OBJINST):
-           /* check for instance of object center point inside area box */
-           selected = (TOOBJINST(curgen)->position.x < areawin->save.x) &&
-		(TOOBJINST(curgen)->position.x > areawin->origin.x) &&
-		(TOOBJINST(curgen)->position.y < areawin->save.y) &&
-		(TOOBJINST(curgen)->position.y > areawin->origin.y);
-	   break;
+	    curinst = TOOBJINST(curgen);
+
+	    /* An object instance is selected if any part of it is	*/
+	    /* selected on a recursive area search.			*/
+            savesave.x = areawin->save.x;
+            savesave.y = areawin->save.y;
+            InvTransformPoints(&areawin->save, &tmppt, 1, curinst->position,
+			curinst->scale, curinst->rotation);
+            areawin->save.x = tmppt.x;
+            areawin->save.y = tmppt.y;
+
+            saveorig.x = areawin->origin.x;
+            saveorig.y = areawin->origin.y;
+            InvTransformPoints(&areawin->origin, &tmppt, 1, curinst->position,
+			curinst->scale, curinst->rotation);
+            areawin->origin.x = tmppt.x;
+            areawin->origin.y = tmppt.y;
+
+            UPushCTM();
+            UPreMultCTM(DCTM, curinst->position, curinst->scale,
+			curinst->rotation);
+            selected = selectarea(curinst->thisobject);
+            UPopCTM();
+            areawin->save.x = savesave.x;
+            areawin->save.y = savesave.y;
+            areawin->origin.x = saveorig.x;
+            areawin->origin.y = saveorig.y;
+	    break;
 
 	case(GRAPHIC):
            /* check for graphic image center point inside area box */
@@ -621,6 +646,9 @@ void selectarea()
 	   /* Ignore temporary labels created by the netlist generator */
 	   if (slab->string->type != FONT_NAME) break;
 
+	   /* Ignore info and pin labels that are not on the top level */
+	   if ((selobj != topobject) && (slab->pin != False)) break;
+
 	   /* create a 4-point polygon out of the select box information */
 	   bboxpts[0].x = bboxpts[3].x = areawin->origin.x;
 	   bboxpts[0].y = bboxpts[1].y = areawin->origin.y;
@@ -694,6 +722,10 @@ void selectarea()
 	   break;
       }
 
+      /* on recursive searches, return as soon as we find something */
+
+      if ((selobj != topobject) && selected) return TRUE;
+
       /* check if this part has already been selected */
 
       if (selected)
@@ -709,6 +741,7 @@ void selectarea()
          *newselect = (short)(curgen - topobject->plist);
       }
    }
+   if (selobj != topobject) return FALSE;
    setoptionmenu();
 
    /* if none or > 1 label has been selected, cancel any textpos placement */
@@ -730,6 +763,7 @@ void selectarea()
       XcInternalTagCall(xcinterp, 2, "select", "here");
 #endif
 
+   return selected;
 }
 
 /*------------------------*/
